use strict;
use warnings;
package HTML::MasonX::Free;
$HTML::MasonX::Free::VERSION = '0.006';
# ABSTRACT: a secret society of HTML::Mason-using throwback modules

#pod =head1 OVERVIEW
#pod
#pod These modules, together, help make HTML::Mason work more like RJBS thinks it
#pod should have always worked.  More documentation about the general ideas will be
#pod here later, when RJBS is not pressed for time.
#pod
#pod =cut

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

HTML::MasonX::Free - a secret society of HTML::Mason-using throwback modules

=head1 VERSION

version 0.006

=head1 OVERVIEW

These modules, together, help make HTML::Mason work more like RJBS thinks it
should have always worked.  More documentation about the general ideas will be
here later, when RJBS is not pressed for time.

=head1 AUTHOR

Ricardo Signes <rjbs@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2016 by Ricardo Signes.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
